<?php
declare(strict_types=1);

namespace FiloBlu\ExtInventory\Model\Algorithms;

use Magento\InventorySourceSelectionApi\Api\Data\InventoryRequestInterface;
use Magento\InventorySourceSelectionApi\Api\Data\SourceSelectionItemInterfaceFactory;
use Magento\InventorySourceSelectionApi\Api\Data\SourceSelectionResultInterface;
use Magento\InventorySourceSelectionApi\Api\Data\SourceSelectionResultInterfaceFactory;
use Magento\InventorySourceSelectionApi\Model\GetInStockSourceItemsBySkusAndSortedSource;
use Magento\InventorySourceSelectionApi\Model\GetSourceItemQtyAvailableInterface;
use Magento\InventorySourceSelectionApi\Model\SourceSelectionInterface;

/**
 * Class FinalSourceBasedAlgorithm
 * @package FiloBlu\ExtInventory\Model\Algorithms
 */
class FinalSourceBasedAlgorithm implements SourceSelectionInterface
{
    /**
     * @var SourceSelectionResultInterfaceFactory
     */
    private $sourceSelectionResultFactory;
    /**
     * @var GetSourceItemQtyAvailableInterface
     */
    private $getSourceItemQtyAvailable;
    /**
     * @var GetInStockSourceItemsBySkusAndSortedSource
     */
    private $getInStockSourceItemsBySkusAndSortedSource;
    /**
     * @var SourceSelectionItemInterfaceFactory
     */
    private $sourceSelectionItemFactory;

    /**
     * FinalSourceBasedAlgorithm constructor.
     * @param SourceSelectionResultInterfaceFactory $sourceSelectionResultFactory
     * @param GetSourceItemQtyAvailableInterface $getSourceItemQtyAvailable
     * @param GetInStockSourceItemsBySkusAndSortedSource $getInStockSourceItemsBySkusAndSortedSource
     * @param SourceSelectionItemInterfaceFactory $sourceSelectionItemFactory
     */
    public function __construct(
        SourceSelectionResultInterfaceFactory $sourceSelectionResultFactory,
        GetSourceItemQtyAvailableInterface $getSourceItemQtyAvailable,
        GetInStockSourceItemsBySkusAndSortedSource $getInStockSourceItemsBySkusAndSortedSource,
        SourceSelectionItemInterfaceFactory $sourceSelectionItemFactory
    ) {
        $this->sourceSelectionResultFactory = $sourceSelectionResultFactory;
        $this->getSourceItemQtyAvailable = $getSourceItemQtyAvailable;
        $this->getInStockSourceItemsBySkusAndSortedSource = $getInStockSourceItemsBySkusAndSortedSource;
        $this->sourceSelectionItemFactory = $sourceSelectionItemFactory;
    }

    /**
     * @param InventoryRequestInterface $inventoryRequest
     * @return SourceSelectionResultInterface
     */
    public function execute(InventoryRequestInterface $inventoryRequest): SourceSelectionResultInterface
    {
        $registry = \Magento\Framework\App\ObjectManager::getInstance()->get(\Magento\Framework\Registry::class);
        $isShippable = true;

        $itemsTdDeliver = [];
        foreach ($inventoryRequest->getItems() as $item) {
            $itemsTdDeliver[$item->getSku()] = $item->getQty();
        }

        $extensionAttributes = $inventoryRequest->getExtensionAttributes();
        $order = $extensionAttributes->getOrder();
        $sortedSourceCodes = [];
        foreach ($order->getItems() as $orderItem) {
            $sku = $orderItem->getSku();
            if (!array_key_exists($sku, $itemsTdDeliver)) {
                continue;
            }
            $extensionAttributes = $orderItem->getExtensionAttributes();
            $filoBluOrderItem = $extensionAttributes->getFilobluOrderItem();
            foreach ($filoBluOrderItem as $info) {
                $sortedSourceCodes[] = $info->getFinalInventorySource()->getSourceCode();
            }
        }

        $sourceItems = $this->getInStockSourceItemsBySkusAndSortedSource->execute(
            array_keys($itemsTdDeliver),
            array_unique($sortedSourceCodes)
        );

        $sourceItemSelections = [];
        foreach ($sourceItems as $sourceItem) {
            $sourceItemQtyAvailable = $this->getSourceItemQtyAvailable->execute($sourceItem);
            $qtyToDeduct = min($sourceItemQtyAvailable, $itemsTdDeliver[$sourceItem->getSku()] ?? 0.0);

            $sourceItemSelections[] = $this->sourceSelectionItemFactory->create([
                'sourceCode'   => $sourceItem->getSourceCode(),
                'sku'          => $sourceItem->getSku(),
                'qtyToDeduct'  => $qtyToDeduct,
                'qtyAvailable' => $sourceItemQtyAvailable
            ]);

            $itemsTdDeliver[$sourceItem->getSku()] -= $qtyToDeduct;
        }

        return $this->sourceSelectionResultFactory->create(
            [
                'sourceItemSelections' => $sourceItemSelections,
                'isShippable'          => $isShippable
            ]
        );
    }
}
